# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class BankDeliveryMethod(str, enum.Enum):
    ACH_STANDARD = "ACH_STANDARD"
    ACH_SAME_DAY = "ACH_SAME_DAY"

    def visit(
        self, ach_standard: typing.Callable[[], T_Result], ach_same_day: typing.Callable[[], T_Result]
    ) -> T_Result:
        if self is BankDeliveryMethod.ACH_STANDARD:
            return ach_standard()
        if self is BankDeliveryMethod.ACH_SAME_DAY:
            return ach_same_day()
