# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class EntityOnboardingLinkType(str, enum.Enum):
    PAYEE = "PAYEE"
    PAYOR = "PAYOR"

    def visit(self, payee: typing.Callable[[], T_Result], payor: typing.Callable[[], T_Result]) -> T_Result:
        if self is EntityOnboardingLinkType.PAYEE:
            return payee()
        if self is EntityOnboardingLinkType.PAYOR:
            return payor()
