# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .codat_provider_request import CodatProviderRequest


class ExternalAccountingSystemProviderRequest_Codat(CodatProviderRequest):
    type: typing_extensions.Literal["codat"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


ExternalAccountingSystemProviderRequest = typing.Union[ExternalAccountingSystemProviderRequest_Codat]
