# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .bank_account_payment_source_options import BankAccountPaymentSourceOptions


class PaymentSourceOptions_BankAccount(BankAccountPaymentSourceOptions):
    type: typing_extensions.Literal["bankAccount"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


PaymentSourceOptions = typing.Union[PaymentSourceOptions_BankAccount]
