# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class CheckDeliveryMethod(str, enum.Enum):
    PRINT = "PRINT"
    MAIL = "MAIL"

    def visit(self, print: typing.Callable[[], T_Result], mail: typing.Callable[[], T_Result]) -> T_Result:
        if self is CheckDeliveryMethod.PRINT:
            return print()
        if self is CheckDeliveryMethod.MAIL:
            return mail()
