# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .check_payment_destination_options import CheckPaymentDestinationOptions


class PaymentDestinationOptions_Check(CheckPaymentDestinationOptions):
    type: typing_extensions.Literal["check"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


PaymentDestinationOptions = typing.Union[PaymentDestinationOptions_Check]
