# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class EmailSenderProvider(str, enum.Enum):
    NONE = "none"
    SENDGRID = "sendgrid"
    SES = "ses"
    RESEND = "resend"

    def visit(
        self,
        none: typing.Callable[[], T_Result],
        sendgrid: typing.Callable[[], T_Result],
        ses: typing.Callable[[], T_Result],
        resend: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is EmailSenderProvider.NONE:
            return none()
        if self is EmailSenderProvider.SENDGRID:
            return sendgrid()
        if self is EmailSenderProvider.SES:
            return ses()
        if self is EmailSenderProvider.RESEND:
            return resend()
