# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class VendorNetwork(str, enum.Enum):
    ALL = "all"
    PLATFORM = "platform"
    ENTITY = "entity"

    def visit(
        self,
        all: typing.Callable[[], T_Result],
        platform: typing.Callable[[], T_Result],
        entity: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is VendorNetwork.ALL:
            return all()
        if self is VendorNetwork.PLATFORM:
            return platform()
        if self is VendorNetwork.ENTITY:
            return entity()
