# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ApproverAction(str, enum.Enum):
    NONE = "NONE"
    APPROVE = "APPROVE"
    REJECT = "REJECT"

    def visit(
        self,
        none: typing.Callable[[], T_Result],
        approve: typing.Callable[[], T_Result],
        reject: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ApproverAction.NONE:
            return none()
        if self is ApproverAction.APPROVE:
            return approve()
        if self is ApproverAction.REJECT:
            return reject()
