# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class PaymentRailMarkupType(str, enum.Enum):
    FLAT = "flat"
    PERCENT = "percent"

    def visit(self, flat: typing.Callable[[], T_Result], percent: typing.Callable[[], T_Result]) -> T_Result:
        if self is PaymentRailMarkupType.FLAT:
            return flat()
        if self is PaymentRailMarkupType.PERCENT:
            return percent()
