# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .approver_rule import ApproverRule


class Rule_Approver(ApproverRule):
    type: typing_extensions.Literal["approver"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


Rule = typing.Union[Rule_Approver]
